/*
 * File         : SOCK_Acceptor.cpp
 * Author       : Tomasz Stachowiak
 * Created      : Wed May 20 15:44:20 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <BVN/BVN.h>
#include <BVN/SOCK_Acceptor.h>
#include <BVN/SAP.h>
#include <BVN/SOCK_Stream.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

int SOCK_Acceptor::open(PEER_ADDR *addr)
{
  if(sap.open(ntohs(addr->sin_port)) == -1)
    {
      BVN_ERROR("SOCK_Acceptor: open\n");
      return -1;
    }

  return 0;
}

int SOCK_Acceptor::close()
{
  return sap.close();
}

int SOCK_Acceptor::accept(PEER_STREAM *stream)
{
  PEER_ADDR addr;
  HANDLE str_handle;
  int addrlen = sizeof(addr);

  HANDLE srv_handle = sap.get_handle();

  if(srv_handle <= 0)
    {
      BVN_ERROR("SOCK_Acceptor: accept, wrong handle\n");
      return -1;
    }

  str_handle = ::accept(srv_handle, &addr, &addrlen);

  if(str_handle == -1)
    {
      BVN_ERROR("SOCK_Acceptor: accept, socket accept\n");
      return -1;
    }

  stream->set_peer_address(&addr);
  stream->set_handle(str_handle);

  return 0;
}

HANDLE SOCK_Acceptor::get_handle(void)
{
  return sap.get_handle();
}
